@@ -1,17 +1,21 @@
 /*
-@version: 0.4
+@version: 0.5
 @author: Sudheer K. <scifi1947 at gmail.com>
 @license: GNU General Public License
 */
 
 
 #include <QtGui/QApplication>
-#include "configwindow.h"
+#include "profileswindow.h"
+#include "vicarprofiletypes.h"
+
 int main(int argc, char *argv[])
 {
+    //qDBusRegisterMetaType<org::maemo::vicar::Profile>();
+    //qDBusRegisterMetaType<org::maemo::vicar::ProfileList>();
 
     QApplication a(argc, argv);            
-    ConfigWindow w;
+    ProfilesWindow w;
     w.show();
     return a.exec();
 }
