@@ -1,5 +1,5 @@
 /*
-@version: 0.4
+@version: 0.5
 @author: Sudheer K. <scifi1947 at gmail.com>
 @license: GNU General Public License
 */
@@ -8,34 +8,46 @@
 #include "vicardbusadaptor.h"
 #include <dbusutility.h>
 #include <gconfutility.h>
+#include <databaseutility.h>
+#include <telepathyutility.h>
 #include <QDebug>
 #include <QRegExp>
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QStringListIterator>
 
-static QString strLastDialedNumber = QString();
+
+//static QString strLastDialedNumber = QString();
+//static org::maemo::vicar::Profile currentProfile;
 
 class CallRouterPrivate
 {
 public:
     CallRouterPrivate(CallRouter * p) :
+        databaseUtility(new DatabaseUtility(p)),
         dbusAdaptor(new VicarDbusAdaptor(p)),
+        dbusUtility(new DbusUtility(p)),        
         gconfUtility(new GConfUtility(p)),
-        dbusUtility(new DbusUtility(p)),
+        tpUtility(new TelepathyUtility(p)),
         parent(p)
     {
         Q_ASSERT(0 != dbusAdaptor);
+        databaseUtility->openDatabase();
     }
 
     ~CallRouterPrivate()
     {
         qDebug() << "VICaR: Call Router Destructing";
+        databaseUtility->closeDatabase();
     }
 
     VicarDbusAdaptor * dbusAdaptor;
-    GConfUtility * gconfUtility;
+    DatabaseUtility *databaseUtility;
     DbusUtility * dbusUtility;
+    GConfUtility * gconfUtility;
+    TelepathyUtility *tpUtility;
+    QString strLastDialedNumber;
+    org::maemo::vicar::Profile *currentProfile;
     CallRouter * const parent;
 };
 
@@ -69,15 +81,11 @@ void CallRouter::registerDBusService(){
         exit(2);
     }
 
-    //this->connectToDBusSignals();
-
 }
 
 
 void CallRouter::unregisterDBusService(){
 
-    //this->disconnectFromDBusSignals();
-
     QDBusConnection connection = d->dbusUtility->getConnection();
 
     connection.unregisterObject(APPLICATION_DBUS_PATH,QDBusConnection::UnregisterTree);
@@ -89,17 +97,39 @@ void CallRouter::unregisterDBusService(){
 
 }
 
-QString CallRouter::callViaCallingCard(){
+QString CallRouter::callViaCallingCard(QString strDestinationNumber){
+
+        d->currentProfile = new org::maemo::vicar::Profile();
+        d->currentProfile->profileID = 0;
+        bool result = d->databaseUtility->findProfileByNumber(strDestinationNumber,d->currentProfile);
+        QString strErrorMessage;
+        if (!result){
+            strErrorMessage = QString("Error finding VICaR profile. %1").arg(d->databaseUtility->lastError().text());
+            return strErrorMessage;
+        }
+        else if (d->currentProfile->profileID == 0){
+            bool routeOnDefault = d->gconfUtility->getGconfValueBoolean("route_on_default");
+            if (routeOnDefault){
+                qDebug() << "Routing directly as per configuration";
+               this->placeCall(strDestinationNumber);
+               return strErrorMessage;
+            }
+            else{
+                qDebug() << "No profile found. Stopping..";
+                strErrorMessage  = "VICaR: No routing profile defined for this number.";
+                d->dbusUtility->displayNotification(strErrorMessage );
+                return strErrorMessage;
+            }
+        }
+
         //Now call the calling card number. This is generally a local and/or tollfree number
-
-        QString strCallingCardNumber = d->gconfUtility->getGconfValueString("calling_card_number");
-
+        QString strCallingCardNumber = d->currentProfile->gatewayNumber;
         qDebug() << "Initiating call to "<< strCallingCardNumber;
-
         bool status = this->placeCall(strCallingCardNumber);
+        d->strLastDialedNumber = strDestinationNumber;
 
         QString strUserMessage;
-        QString strErrorMessage;
+
         if (status){
             qDebug() << "Call initiated successfully. Connecting DBus slot for audio connection monitor";
              startCallStatusMonitors();
@@ -108,7 +138,9 @@ QString CallRouter::callViaCallingCard(){
             strUserMessage = QString("Unable to initiate new call to ").append(strCallingCardNumber);
             strErrorMessage = d->dbusUtility->getErrorMessage();
             qDebug() << strErrorMessage;
-            strLastDialedNumber.clear();
+            d->strLastDialedNumber.clear();
+            delete d->currentProfile;
+            d->currentProfile = 0;
         }
 
         d->dbusUtility->displayNotification(strUserMessage);
@@ -194,7 +226,9 @@ void CallRouter::startCallStatusMonitors(){
 
 void CallRouter::stopCallStatusMonitors(){
 
-    strLastDialedNumber.clear();
+    d->strLastDialedNumber.clear();
+    delete d->currentProfile;
+    d->currentProfile = 0;
 
     QDBusConnection connection = d->dbusUtility->getConnection();
 
@@ -246,7 +280,7 @@ void CallRouter::stopCallStatusMonitors(){
 
 void CallRouter::sendNumberAsDTMFCode(const QDBusMessage& dbusMessage){
 
-    if (!strLastDialedNumber.isEmpty()){
+    if (!d->strLastDialedNumber.isEmpty() && d->currentProfile != 0){
         //Verify whether we have the last dialed number available
 
         QList<QVariant> listArguments = dbusMessage.arguments();
@@ -254,7 +288,7 @@ void CallRouter::sendNumberAsDTMFCode(const QDBusMessage& dbusMessage){
 
         if (audioConnected){
             // Now that the call to Calling card number is successful. We can send the original number as DTMF tones
-            QString strDTMFCode = convertToDTMFCode(strLastDialedNumber);
+            QString strDTMFCode = convertToDTMFCode(d->strLastDialedNumber);
 
             qDebug() << "Audio connection established. Sending DTMF code "<< strDTMFCode;
 
@@ -285,6 +319,9 @@ void CallRouter::sendNumberAsDTMFCode(const QDBusMessage& dbusMessage){
             qDebug() << "Now disconnecting from call status monitors..";
             stopCallStatusMonitors();
 
+            d->strLastDialedNumber.clear();
+            delete d->currentProfile;
+            d->currentProfile = 0;
         }
         else{
             qDebug() << "Audio not yet connected.";
@@ -300,24 +337,20 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
     QString strDTMFCode;
 
     if (!strNumber.isEmpty()){
-
-        //int intDTMFDelay = d->gconfUtility->getGconfValueInteger("dtmf_delay");
-
-        //if (intDTMFDelay <1 ) intDTMFDelay = 1;
-
         int intDTMFDelay = 1;
 
         //Add the prefix p so that there is some delay after the call is picked up by the automated system to send DTMF tones.
         strDTMFCode = QString("").fill('p',intDTMFDelay);        
 
         //Now check whether we need a prefix
-        QString strDTMFPrefix = d->gconfUtility->getGconfValueString("dtmf_prefix");
+        QString strDTMFPrefix = d->currentProfile->dtmfPrefix;
+
         if (!strDTMFPrefix.isEmpty()){
             strDTMFCode = strDTMFCode.append(strDTMFPrefix);
         }
 
         //Get the format required by calling card from coniguration
-        QString qstrDTMFFormat = d->gconfUtility->getGconfValueString("dtmf_format");
+        QString qstrDTMFFormat = d->currentProfile->dtmfFormat;
         if (qstrDTMFFormat.isEmpty()) qstrDTMFFormat = "<Country Code><Area Code><Phone Number>";
 
         /* Replace 00 (international dialing code) at the beginning
@@ -342,8 +375,8 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
         strDTMFCode = strDTMFCode.append(strNumber);
 
         //Now check whether we need a suffix
-        QString strDTMFSuffix = d->gconfUtility->getGconfValueString("dtmf_suffix");
-        if (!strDTMFSuffix.isEmpty() && !strDTMFSuffix.contains("--None--")){
+        QString strDTMFSuffix = d->currentProfile->dtmfSuffix;
+        if (!strDTMFSuffix.isEmpty()){
             strDTMFCode = strDTMFCode.append(strDTMFSuffix);
         }
     }
@@ -351,81 +384,33 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
     return strDTMFCode;
 }
 
-bool CallRouter::isExcludedNumber(QString strInternationalNumber){
-
-    bool isExcluded = false;
-
-    //Get the list of excluded codes
-    QString qstrExcludedNumbers = d->gconfUtility->getGconfValueString("numbers_to_exclude");
-    QStringList strExcludedCodeList = qstrExcludedNumbers.split(",");
-    QStringListIterator iterator(strExcludedCodeList);
-
-    QRegExp regexp = QRegExp("(^0{2})|[^0-9p]");
-
-    while (iterator.hasNext()){
-        QString strCode = iterator.next();
-        strCode = strCode.replace(regexp,"");
-        strInternationalNumber = strInternationalNumber.replace(regexp,"");
-        if (!strCode.isEmpty() && strInternationalNumber.startsWith(strCode)){
-            isExcluded = true;
-        }
-    }
-    return isExcluded;
-}
-
 //DBus Method used by external applications to check whether VICaR is enabled and running
 bool CallRouter::isRunning(){
 
-    //Verify Whether Call Routing is Enabled
-    bool isRoutingEnabled = d->gconfUtility->getGconfValueBoolean("routing_enabled");
-    if (isRoutingEnabled){
+    return true;
+
+    //Verify Whether VICaR telepathy account is online
+    /*
+    if (d->tpUtility->getAccountStatus() == "Connected"){
         return true;
     }
     else{
         return false;
     }
+    */
 }
 
 //DBus Method used by external applications to call via VICaR
-QString CallRouter::callInternationalNumber(const QString& strInternationalNumber){
+QString CallRouter::callInternationalNumber(const QString& strDestinationNumber){
 
-    //QDBusConnection connection = d->dbusUtility->getConnection();
-    QString strReturnMessage = "";
+    qDebug() << "New call requested by external application. Destination number is " << strDestinationNumber;
+    QString strErrorMessage = this->callViaCallingCard(strDestinationNumber);
+    qDebug() << strErrorMessage;
 
-    qDebug() << "New call requested by external application. Destination number is " << strInternationalNumber;
-
-     if (strInternationalNumber.startsWith("+") ||
-         strInternationalNumber.startsWith("00"))
-     {
-         qDebug() << "International number "<< strInternationalNumber << " recognized. Starting proceedings..";
-
-         //Check whether this is one of the excluded country codes
-         if (!isExcludedNumber(strInternationalNumber)){
-             strLastDialedNumber = strInternationalNumber;
-             QString strErrorMessage = this->callViaCallingCard();
-             if (!strErrorMessage.isEmpty()){
-                 strReturnMessage = strErrorMessage;
-             }
-         }
-         else{
-             //Country code is excluded, place a call directly
-             qDebug() << "Country code is excluded, placing call directly";
-             bool status = this->placeCall(strInternationalNumber);
-             if (!status){
-                 strReturnMessage = d->dbusUtility->getErrorMessage();
-             }
-         }
-    }
-    else{
-        strReturnMessage = "VICaR Error: Phone number is not in international format. Use +123456789 format.";
-    }
-
-    qDebug() << strReturnMessage;
-
-    if (strReturnMessage.isEmpty()){
+    if (strErrorMessage.isEmpty()){
         return QString("Success");
     }
     else{
-        return strReturnMessage;
+        return strErrorMessage;
     }
  }
