@@ -1,5 +1,5 @@
 /*
-@version: 0.4
+@version: 0.5
 @author: Sudheer K. <scifi1947 at gmail.com>
 @license: GNU General Public License
 
@@ -90,7 +90,9 @@ int main(int argc, char ** argv)
     qDBusRegisterMetaType<org::freedesktop::Telepathy::CapabilityPair>();
     qDBusRegisterMetaType<org::freedesktop::Telepathy::CapabilityPairList>();
     qDBusRegisterMetaType<org::freedesktop::Telepathy::CapabilityChange>();
-    qDBusRegisterMetaType<org::freedesktop::Telepathy::CapabilityChangeList>();
+    qDBusRegisterMetaType<org::freedesktop::Telepathy::CapabilityChangeList>();    
+    qDBusRegisterMetaType<org::freedesktop::Telepathy::RequestableChannelClass>();
+    qDBusRegisterMetaType<org::freedesktop::Telepathy::RequestableChannelClassList>();
 
     QDBusConnection connection = QDBusConnection::sessionBus();
 
